# from pesto.common.testing.endpoint_manager import EndpointChecker
#
#
# def test_endpoint_checker():
#     checker = EndpointChecker(
#         server_url='http://localhost:4000'
#     )
#     checker.test_with('resources/data')
